	SUBROUTINE CD_DEFER_COORD_WRITE(cdaxis, grid, idim, lo, hi,
     .				        axis_type)


*  This software was developed by the Thermal Modeling and Analysis
*  Project(TMAP) of the National Oceanographic and Atmospheric
*  Administration's (NOAA) Pacific Marine Environmental Lab(PMEL),
*  hereafter referred to as NOAA/PMEL/TMAP.
*
*  Access and use of this software shall impose the following
*  obligations and understandings on the user. The user is granted the
*  right, without any fee or cost, to use, copy, modify, alter, enhance
*  and distribute this software, and any derivative works thereof, and
*  its supporting documentation for any purpose whatsoever, provided
*  that this entire notice appears in all copies of the software,
*  derivative works and supporting documentation.  Further, the user
*  agrees to credit NOAA/PMEL/TMAP in any publications that result from
*  the use of this software or in any product that includes this
*  software. The names TMAP, NOAA and/or PMEL, however, may not be used
*  in any advertising or publicity to endorse or promote any products
*  or commercial entity unless specific written permission is obtained
*  from NOAA/PMEL/TMAP. The user also understands that NOAA/PMEL/TMAP
*  is not obligated to provide the user with any support, consulting,
*  training or assistance of any kind with regard to the use, operation
*  and performance of this software nor to provide the user with any
*  updates, revisions, new versions or "bug fixes".
*
*  THIS SOFTWARE IS PROVIDED BY NOAA/PMEL/TMAP "AS IS" AND ANY EXPRESS
*  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
*  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED. IN NO EVENT SHALL NOAA/PMEL/TMAP BE LIABLE FOR ANY SPECIAL,
*  INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
*  RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
*  CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN
*  CONNECTION WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE. 
*
*
* Set variables to perform writing of netCDF axis coordinates at a later time
* (required for performance since switching between DEFINE and DATA modes
* can be so costly)

* Programmer Steve Hankin
* NOAA/PMEL, Seattle, WA - Tropical Modeling and Analysis Program

* V530 *sh* - 12/00

* argument declarations:
	INTEGER cdaxis, grid, idim, lo, hi, axis_type

* include files
        include 'cdf_tmap.parm'
        include 'xdefer_coords.cmn'
	include 'xio.cmn_text'

* local variable declarations:

	IF ( axis_type .EQ. pcd_defer_init ) THEN
* initialize the list of deferred axes
	  dfr_num = 0

	ELSE
* store info for a deferred axis coord points write
	  dfr_num = dfr_num + 1
	  dfr_cdaxis(dfr_num) = cdaxis
	  dfr_grid(dfr_num) = grid
	  dfr_idim(dfr_num) = idim 
	  dfr_lo(dfr_num) = lo
	  dfr_hi(dfr_num) = hi
	  dfr_axis_type(dfr_num) = axis_type
	ENDIF

	RETURN
	END






