*
* Definition for GOPKS (open GKS).  Does nothing since FGD_GOPWS 
* initializes the appropriate graphics engine if needed.
*

        SUBROUTINE FGD_GOPKS(gkserrlun)

        IMPLICIT NONE
        INTEGER gkserrlun

        RETURN
        END

