*
* Definition for GSDS (set deferral state).  Does nothing since
* update of the display is left to the graphics engine.
*

        SUBROUTINE FGD_GSDS(windowid, deferralmode, regenmode)

        IMPLICIT NONE
        INTEGER windowid, deferralmode, regenmode

        RETURN
        END

