*
* Definition of GSVPIP
* (set viewport input priority)
* Does nothing since input functions are not supported.
*

        SUBROUTINE FGD_GSVPIP(transnum, reftransnum, relpriority)

        IMPLICIT NONE
        INTEGER transnum, reftransnum, relpriority

        RETURN
        END

