*
* Definition of GSLCM
* (set locator mode)
* DOes nothing as input functions are not supported at this time
*

        SUBROUTINE FGD_GSLCM(windowid, locdevnum, opermode, echomode)

        IMPLICIT NONE
        INTEGER windowid, locdevnum, opermode, echomode

        RETURN
        END

