* scat2grid_subs_mask:  Subroutines called by the scat2grid functions
* where a mask is applied.
*
*  This software was developed by the Thermal Modeling and Analysis
*  Project(TMAP) of the National Oceanographic and Atmospheric
*  Administration's (NOAA) Pacific Marine Environmental Lab(PMEL),
*  hereafter referred to as NOAA/PMEL/TMAP.
*
*  Access and use of this software shall impose the following
*  obligations and understandings on the user. The user is granted the
*  right, without any fee or cost, to use, copy, modify, alter, enhance
*  and distribute this software, and any derivative works thereof, and
*  its supporting documentation for any purpose whatsoever, provided
*  that this entire notice appears in all copies of the software,
*  derivative works and supporting documentation.  Further, the user
*  agrees to credit NOAA/PMEL/TMAP in any publications that result from
*  the use of this software or in any product that includes this
*  software. The names TMAP, NOAA and/or PMEL, however, may not be used
*  in any advertising or publicity to endorse or promote any products
*  or commercial entity unless specific written permission is obtained
*  from NOAA/PMEL/TMAP. The user also understands that NOAA/PMEL/TMAP
*  is not obligated to provide the user with any support, consulting,
*  training or assistance of any kind with regard to the use, operation
*  and performance of this software nor to provide the user with any
*  updates, revisions, new versions or "bug fixes".
*
*  THIS SOFTWARE IS PROVIDED BY NOAA/PMEL/TMAP "AS IS" AND ANY EXPRESS
*  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
*  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED. IN NO EVENT SHALL NOAA/PMEL/TMAP BE LIABLE FOR ANY SPECIAL,
*  INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
*  RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
*  CONTRACT, NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN
*  CONNECTION WITH THE ACCESS, USE OR PERFORMANCE OF THIS SOFTWARE. 
*

	SUBROUTINE maskxy (xx, yy, xaxis, yaxis, mask, nx, ny, flag)
	REAL xx, yy, xaxis(nx), yaxis(ny), mask(nx,ny)
	integer flag

	REAL dx2, dy2

	dx2 = (xaxis(2) - xaxis(1))/2
	dy2 = (yaxis(2) - yaxis(1))/2

        flag = 0
	DO 100 j = 1, ny
	   IF (yy.GE.yaxis(j)-dx2 .AND. yy.LT.yaxis(j)+dy2) THEN
	      DO 200 i = 1, nx
	         IF (xx.GE.xaxis(i)-dx2 .AND. xx.LT.xaxis(i)+dx2) THEN
                    IF (mask(i,j) .EQ. 1) THEN
		       flag = 1
		       RETURN
		    ENDIF
		 ENDIF
  200         CONTINUE
           ENDIF
  100   CONTINUE

        RETURN
	END

