*
* Definition of GSASF
* (set aspect source flags)
* Does nothing as these flags are not used.
*

        SUBROUTINE FGD_GSASF(lasf)

        IMPLICIT NONE
        INTEGER lasf(13)

        RETURN
        END
