'''
Empirical-orthogonal-function analysis functions designed for use in PyFerret, 
especially for Python external functions for PyFerret.

@author: Karl Smith
'''

# Import classes given in modules in this package so they are all seen here.
from eofanalysis import InvalidStateError, EOFAnalysis

